/*
 * FILE --- node_list.hh
 *
 * PURPOSE --- Header file for classes that implement a structure of
 * a node for a linked list and the structure of the linked list itself.
 * The node and the linked list are especially designed for an algorithm
 * for implementing a limited lookahed (planning) controller. 
 * The user specifies the dimension of the state of the plant to be controlled.
 *
 */

// Prevent Multiple Inclusion
#ifndef NODE_LIST_HH
#define NODE_LIST_HH

// Include Files
#include <stdlib.h>

// Local Definitions
#define OPEN_NO_ERROR 0
#define OPEN_MEM_ERROR 1

class OPEN_NODE
{
  friend class OPEN_LIST;
public:

  // Constructor
  OPEN_NODE(int _plant_order);
 
  // Destructor
  ~OPEN_NODE();

  double *state;   // State of the current node
  int ctrl;        // The # of the initial control input 
  double cost;     // Cost of coming from the initial to the current 

  // Set the fields of the node
  void set_node(const double *_state, int _ctrl, double _cost);

  // Get the state field of the node
  void get_state(double *_state);

  // Get the cost field of the node
  double get_cost(void) {return cost;};

  // Get the control field of the node
  int get_ctrl(void) {return ctrl;};

  int open_node_err;  // Variable to hold the error condition

  int plant_order; // Dimension of the state vector
  OPEN_NODE *nextPtr; // Pointer to the next node

private:
  // Allocate/free memory for the gains
  void allocate_memory(void);
  void free_memory(void);
};
typedef struct OPEN_NODE open_node;

class OPEN_LIST
{
public:

  // Constructor
  OPEN_LIST();
 
  // Destructor
  ~OPEN_LIST();

  // Add the specified node to the top of the list
  void add_node(open_node *_new_node);

  // Remove the top node of the list and return its state
  // Return value: 0 == error, 1 == success
  int remove_node(double *_state);

  // Check whether the list is empty
  // Return value: 0 == empty, 1 == not empty
  int isEmpty(void) const;

  int num_of_elements; // Num of the elements in the list

  OPEN_NODE *firstPtr; // Pointer to the first node
  OPEN_NODE *lastPtr; // Pointer to the last node
};
typedef struct OPEN_LIST open_list;




#endif // NODE_LIST_HH
