/*
 * FILE --- llp_ex.cc 
 *
 * PURPOSE --- Main source file that provides an example usage of the 
 * Limited Lookahead Policy (LLP) controller utility. The example 
 * constitutes the load balancing problem of a six machine Flexible 
 * Manufacturing System (FMS).
 *
 */

// Include Files
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "fms.hh"

void main(void) 
{
  int ii, jj; // Index variales for the loops
  int num_of_mach = 6; 
  int num_of_psbl_inputs = 10;
  int lookahead_horizon = 3;
  double **ctrl_inputs;
  double *best_ctrl;

  FMS *my_fms;
  
  FILE *fp;  // To save data
  
  // The 10 possible inputs of dimension 6 
  double ctrl_input[6][10] = 
  {{-1,  1, -1,  0,  0,  0,  0,  0,  0,  0},
   { 1, -1,  0,  1,  0,  0,  0,  0,  0,  0},
   { 0,  0,  1,  0, -1,  1, -1,  0,  0,  0},
   { 0,  0,  0, -1,  1, -1,  0,  1,  0,  0},
   { 0,  0,  0,  0,  0,  0,  1,  0, -1,  1},
   { 0,  0,  0,  0,  0,  0,  0, -1,  1, -1}};
  
  // Initial plant state
  double plant_state[6] = {0, 6, 7, 8, 9, 10};
  
  if ((best_ctrl = new double[num_of_mach]) == NULL)
    printf("Memory error!!!!\n");
  
  if ((ctrl_inputs = new double*[num_of_psbl_inputs]) == NULL)
    printf("Memory error!!!!\n");
      
  for (ii = 0; ii < num_of_psbl_inputs; ii++)
    {
      if ((ctrl_inputs[ii] = new double[num_of_mach]) == NULL)
	printf("Memory error!!!!\n");
    }
  
  // Opne the file
  if (!(fp = fopen("llp_out.dat","w"))) 
    {
      printf("Can't open file.\n");
      exit(1);
    }
  
  // I used ctrl_input to visualize. Otherwise, I have to provide
  // The control inputs as (double **) to the system
  for (ii = 0; ii < num_of_psbl_inputs; ii++)
    {
      for (jj = 0; jj < num_of_mach; jj++)
	{
	  ctrl_inputs[ii][jj] = ctrl_input[jj][ii];
	}
    }
  
  // Create the FMS object
  my_fms = new FMS(num_of_mach, num_of_mach, lookahead_horizon, num_of_psbl_inputs);
  
  // Set the control inputs and the initial state
  my_fms->set_ctrl_inputs((const double **)ctrl_inputs);
  my_fms->set_state((const double *) plant_state);
  
  // Simulate until the load is balanced
  jj = 1;
  do {
    // Find the best control
    my_fms->find_ctrl(best_ctrl);

    // Apply the control and move to the next state
    my_fms->dynamics(my_fms->plant_state, best_ctrl, plant_state);
    my_fms->set_state((const double *) plant_state);
    
    // Save the data
    fprintf(fp,"%d: ", jj);
    for (ii = 0; ii < num_of_mach; ii++)
      {
	fprintf(fp, "%f  ", my_fms->plant_state[ii]);
      }
    fprintf(fp,"\n");
    jj++;

  } while (my_fms->find_cost(my_fms->plant_state, best_ctrl) > 0.5);
  
  // Print the final state
  printf("\n The balanced state ...\n");
  for (ii = 0; ii < num_of_mach; ii++)
    {
      printf("%f  ", my_fms->plant_state[ii]);
    }
  printf("\n");
  
  // Free the allocated memory and close the output file
  for (ii = 0; ii < num_of_psbl_inputs; ii++)
    {
      delete [] ctrl_inputs[ii];
    }
  delete [] ctrl_inputs;
  delete [] best_ctrl;
  
  delete my_fms;

  fclose(fp);
}
